@@ -1,5 +1,5 @@
-# identity.py
-# Copyright (C) the SQLAlchemy authors and contributors
+# orm/identity.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -12,51 +12,56 @@ from sqlalchemy.orm import attributes
 
 class IdentityMap(dict):
     def __init__(self):
-        self._mutable_attrs = {}
-        self.modified = False
+        self._mutable_attrs = set()
+        self._modified = set()
         self._wr = weakref.ref(self)
-        
+
+    def replace(self, state):
+        raise NotImplementedError()
+
     def add(self, state):
         raise NotImplementedError()
-    
+
     def remove(self, state):
         raise NotImplementedError()
-    
+
     def update(self, dict):
         raise NotImplementedError("IdentityMap uses add() to insert data")
-    
+
     def clear(self):
         raise NotImplementedError("IdentityMap uses remove() to remove data")
-        
+
     def _manage_incoming_state(self, state):
         state._instance_dict = self._wr
-        
-        if state.modified:  
-            self.modified = True
+
+        if state.modified:
+            self._modified.add(state)
         if state.manager.mutable_attributes:
-            self._mutable_attrs[state] = True
-    
+            self._mutable_attrs.add(state)
+
     def _manage_removed_state(self, state):
         del state._instance_dict
-        
-        if state in self._mutable_attrs:
-            del self._mutable_attrs[state]
-            
+        self._mutable_attrs.discard(state)
+        self._modified.discard(state)
+
+    def _dirty_states(self):
+        return self._modified.union(s for s in self._mutable_attrs.copy()
+                                    if s.modified)
+
     def check_modified(self):
         """return True if any InstanceStates present have been marked as 'modified'."""
-        
-        if not self.modified:
-            for state in list(self._mutable_attrs):
-                if state.check_modified():
-                    return True
-            else:
-                return False
-        else:
+
+        if self._modified:
             return True
-            
+        else:
+            for state in self._mutable_attrs.copy():
+                if state.modified:
+                    return True
+        return False
+
     def has_key(self, key):
         return key in self
-        
+
     def popitem(self):
         raise NotImplementedError("IdentityMap uses remove() to remove data")
 
@@ -74,8 +79,11 @@ class IdentityMap(dict):
 
     def __delitem__(self, key):
         raise NotImplementedError("IdentityMap uses remove() to remove data")
-        
+
 class WeakInstanceDict(IdentityMap):
+    def __init__(self):
+        IdentityMap.__init__(self)
+        self._remove_mutex = base_util.threading.Lock()
 
     def __getitem__(self, key):
         state = dict.__getitem__(self, key)
@@ -97,155 +105,168 @@ class WeakInstanceDict(IdentityMap):
                 return False
         except KeyError:
             return False
-        return o is not None
-    
+        else:
+            return o is not None
+
     def contains_state(self, state):
         return dict.get(self, state.key) is state
-        
+
+    def replace(self, state):
+        if dict.__contains__(self, state.key):
+            existing = dict.__getitem__(self, state.key)
+            if existing is not state:
+                self._manage_removed_state(existing)
+            else:
+                return
+
+        dict.__setitem__(self, state.key, state)
+        self._manage_incoming_state(state)
+
     def add(self, state):
         if state.key in self:
             if dict.__getitem__(self, state.key) is not state:
-                raise AssertionError("A conflicting state is already present in the identity map for key %r" % state.key)
+                raise AssertionError("A conflicting state is already "
+                                    "present in the identity map for key %r" 
+                                    % (state.key, ))
         else:
             dict.__setitem__(self, state.key, state)
             self._manage_incoming_state(state)
-    
+
     def remove_key(self, key):
         state = dict.__getitem__(self, key)
         self.remove(state)
-        
+
     def remove(self, state):
-        if dict.pop(self, state.key) is not state:
-            raise AssertionError("State %s is not present in this identity map" % state)
+        self._remove_mutex.acquire()
+        try:
+            if dict.pop(self, state.key) is not state:
+                raise AssertionError("State %s is not present in this identity map" % state)
+        finally:
+            self._remove_mutex.release()
+
         self._manage_removed_state(state)
-    
+
     def discard(self, state):
         if self.contains_state(state):
             dict.__delitem__(self, state.key)
             self._manage_removed_state(state)
-        
+
     def get(self, key, default=None):
-        try:
-            return self[key]
-        except KeyError:
+        state = dict.get(self, key, default)
+        if state is default:
             return default
-            
+        o = state.obj()
+        if o is None:
+            o = state._is_really_none()
+        if o is None:
+            return default
+        return o
+
+
     def items(self):
+    # Py2K
         return list(self.iteritems())
 
     def iteritems(self):
-        for state in dict.itervalues(self):
-            value = state.obj()
-            if value is not None:
-                yield state.key, value
+    # end Py2K
+        self._remove_mutex.acquire()
+        try:
+            result = []
+            for state in dict.values(self):
+                value = state.obj()
+                if value is not None:
+                    result.append((state.key, value))
 
-    def itervalues(self):
-        for state in dict.itervalues(self):
-            instance = state.obj()
-            if instance is not None:
-                yield instance
+            return iter(result)
+        finally:
+            self._remove_mutex.release()
 
     def values(self):
+    # Py2K
         return list(self.itervalues())
 
+    def itervalues(self):
+    # end Py2K
+        self._remove_mutex.acquire()
+        try:
+            result = []
+            for state in dict.values(self):
+                value = state.obj()
+                if value is not None:
+                    result.append(value)
+
+            return iter(result)
+        finally:
+            self._remove_mutex.release()
+
     def all_states(self):
-        return dict.values(self)
-    
+        self._remove_mutex.acquire()
+        try:
+            # Py3K
+            # return list(dict.values(self))
+
+            # Py2K
+            return dict.values(self)
+            # end Py2K
+        finally:
+            self._remove_mutex.release()
+
     def prune(self):
         return 0
-        
+
 class StrongInstanceDict(IdentityMap):
     def all_states(self):
-        return [attributes.instance_state(o) for o in self.values()]
-    
+        return [attributes.instance_state(o) for o in self.itervalues()]
+
     def contains_state(self, state):
         return state.key in self and attributes.instance_state(self[state.key]) is state
-    
-    def add(self, state):
+
+    def replace(self, state):
+        if dict.__contains__(self, state.key):
+            existing = dict.__getitem__(self, state.key)
+            existing = attributes.instance_state(existing)
+            if existing is not state:
+                self._manage_removed_state(existing)
+            else:
+                return
+
         dict.__setitem__(self, state.key, state.obj())
         self._manage_incoming_state(state)
-    
+
+    def add(self, state):
+        if state.key in self:
+            if attributes.instance_state(dict.__getitem__(self, state.key)) is not state:
+                raise AssertionError("A conflicting state is already present in the identity map for key %r" % (state.key, ))
+        else:
+            dict.__setitem__(self, state.key, state.obj())
+            self._manage_incoming_state(state)
+
     def remove(self, state):
-        if dict.pop(self, state.key) is not state:
+        if attributes.instance_state(dict.pop(self, state.key)) is not state:
             raise AssertionError("State %s is not present in this identity map" % state)
         self._manage_removed_state(state)
-    
+
     def discard(self, state):
         if self.contains_state(state):
             dict.__delitem__(self, state.key)
             self._manage_removed_state(state)
-            
+
     def remove_key(self, key):
-        state = dict.__getitem__(self, key)
+        state = attributes.instance_state(dict.__getitem__(self, key))
         self.remove(state)
 
     def prune(self):
         """prune unreferenced, non-dirty states."""
-        
+
         ref_count = len(self)
-        dirty = [s.obj() for s in self.all_states() if s.check_modified()]
-        keepers = weakref.WeakValueDictionary(self)
+        dirty = [s.obj() for s in self.all_states() if s.modified]
+
+        # work around http://bugs.python.org/issue6149
+        keepers = weakref.WeakValueDictionary()
+        keepers.update(self)
+
         dict.clear(self)
         dict.update(self, keepers)
         self.modified = bool(dirty)
         return ref_count - len(self)
-        
-class IdentityManagedState(attributes.InstanceState):
-    def _instance_dict(self):
-        return None
-    
-    def modified_event(self, attr, should_copy, previous, passive=False):
-        attributes.InstanceState.modified_event(self, attr, should_copy, previous, passive)
-        
-        instance_dict = self._instance_dict()
-        if instance_dict:
-            instance_dict.modified = True
-    
-    def _is_really_none(self):
-        """do a check modified/resurrect.
-        
-        This would be called in the extremely rare
-        race condition that the weakref returned None but
-        the cleanup handler had not yet established the 
-        __resurrect callable as its replacement.
-        
-        """
-        if self.check_modified():
-            self.obj = self.__resurrect
-            return self.obj()
-        else:
-            return None
-            
-    def _cleanup(self, ref):
-        """weakref callback.
-        
-        This method may be called by an asynchronous
-        gc.
-        
-        If the state shows pending changes, the weakref
-        is replaced by the __resurrect callable which will
-        re-establish an object reference on next access,
-        else removes this InstanceState from the owning
-        identity map, if any.
-        
-        """
-        if self.check_modified():
-            self.obj = self.__resurrect
-        else:
-            instance_dict = self._instance_dict()
-            if instance_dict:
-                instance_dict.remove(self)
-            self.dispose()
-            
-    def __resurrect(self):
-        """A substitute for the obj() weakref function which resurrects."""
-        
-        # store strong ref'ed version of the object; will revert
-        # to weakref when changes are persisted
-        obj = self.manager.new_instance(state=self)
-        self.obj = weakref.ref(obj, self._cleanup)
-        self._strong_obj = obj
-        obj.__dict__.update(self.dict)
-        self.dict = obj.__dict__
-        self._run_on_load(obj)
-        return obj
+
