@@ -1,11 +1,22 @@
+# sql/operators.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """Defines operators used in SQL expressions."""
 
 from operator import (
-    and_, or_, inv, add, mul, sub, div, mod, truediv, lt, le, ne, gt, ge, eq
+    and_, or_, inv, add, mul, sub, mod, truediv, lt, le, ne, gt, ge, eq, neg
     )
+
+# Py2K
+from operator import (div,)
+# end Py2K
+
 from sqlalchemy.util import symbol
 
 
@@ -78,18 +89,26 @@ def desc_op(a):
 def asc_op(a):
     return a.asc()
 
+
 _commutative = set([eq, ne, add, mul])
 def is_commutative(op):
     return op in _commutative
 
+_associative = _commutative.union([concat_op, and_, or_])
+
+
 _smallest = symbol('_smallest')
 _largest = symbol('_largest')
 
 _PRECEDENCE = {
     from_: 15,
     mul: 7,
+    truediv: 7,
+    # Py2K
     div: 7,
+    # end Py2K
     mod: 7,
+    neg: 7,
     add: 6,
     sub: 6,
     concat_op: 6,
@@ -114,7 +133,7 @@ _PRECEDENCE = {
     and_: 3,
     or_: 2,
     comma_op: -1,
-    collate: -2,
+    collate: 7,
     as_: -1,
     exists: 0,
     _smallest: -1000,
@@ -122,5 +141,8 @@ _PRECEDENCE = {
 }
 
 def is_precedent(operator, against):
-    return (_PRECEDENCE.get(operator, _PRECEDENCE[_smallest]) <=
+    if operator is against and operator in _associative:
+        return False
+    else:
+        return (_PRECEDENCE.get(operator, _PRECEDENCE[_smallest]) <=
             _PRECEDENCE.get(against, _PRECEDENCE[_largest]))
