@@ -1,3 +1,9 @@
+# sql/visitors.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 """Visitor/traversal interface and library functions.
 
 SQLAlchemy schema and expression constructs rely on a Python-centric
@@ -26,40 +32,38 @@ __all__ = ['VisitableType', 'Visitable', 'ClauseVisitor',
     'CloningVisitor', 'ReplacingCloningVisitor', 'iterate', 
     'iterate_depthfirst', 'traverse_using', 'traverse',
     'cloned_traverse', 'replacement_traverse']
-    
+
 class VisitableType(type):
     """Metaclass which checks for a `__visit_name__` attribute and
     applies `_compiler_dispatch` method to classes.
-    
+
     """
-    
+
     def __init__(cls, clsname, bases, clsdict):
-        if cls.__name__ == 'Visitable':
+        if cls.__name__ == 'Visitable' or not hasattr(cls, '__visit_name__'):
             super(VisitableType, cls).__init__(clsname, bases, clsdict)
             return
-        
-        assert hasattr(cls, '__visit_name__'), "`Visitable` descendants " \
-                                               "should define `__visit_name__`"
-        
+
         # set up an optimized visit dispatch function
         # for use by the compiler
-        visit_name = cls.__visit_name__
-        if isinstance(visit_name, str):
-            getter = operator.attrgetter("visit_%s" % visit_name)
-            def _compiler_dispatch(self, visitor, **kw):
-                return getter(visitor)(self, **kw)
-        else:
-            def _compiler_dispatch(self, visitor, **kw):
-                return getattr(visitor, 'visit_%s' % self.__visit_name__)(self, **kw)
-    
-        cls._compiler_dispatch = _compiler_dispatch
-        
+        if '__visit_name__' in cls.__dict__:
+            visit_name = cls.__visit_name__
+            if isinstance(visit_name, str):
+                getter = operator.attrgetter("visit_%s" % visit_name)
+                def _compiler_dispatch(self, visitor, **kw):
+                    return getter(visitor)(self, **kw)
+            else:
+                def _compiler_dispatch(self, visitor, **kw):
+                    return getattr(visitor, 'visit_%s' % self.__visit_name__)(self, **kw)
+
+            cls._compiler_dispatch = _compiler_dispatch
+
         super(VisitableType, cls).__init__(clsname, bases, clsdict)
 
 class Visitable(object):
     """Base class for visitable objects, applies the
     ``VisitableType`` metaclass.
-    
+
     """
 
     __metaclass__ = VisitableType
@@ -67,27 +71,27 @@ class Visitable(object):
 class ClauseVisitor(object):
     """Base class for visitor objects which can traverse using 
     the traverse() function.
-    
+
     """
-    
+
     __traverse_options__ = {}
-    
-    def traverse_single(self, obj):
+
+    def traverse_single(self, obj, **kw):
         for v in self._visitor_iterator:
             meth = getattr(v, "visit_%s" % obj.__visit_name__, None)
             if meth:
-                return meth(obj)
-    
+                return meth(obj, **kw)
+
     def iterate(self, obj):
         """traverse the given expression structure, returning an iterator of all elements."""
 
         return iterate(obj, self.__traverse_options__)
-        
+
     def traverse(self, obj):
         """traverse and visit the given expression structure."""
 
         return traverse(obj, self.__traverse_options__, self._visitor_dict)
-    
+
     @util.memoized_property
     def _visitor_dict(self):
         visitors = {}
@@ -96,11 +100,11 @@ class ClauseVisitor(object):
             if name.startswith('visit_'):
                 visitors[name[6:]] = getattr(self, name)
         return visitors
-        
+
     @property
     def _visitor_iterator(self):
         """iterate through this visitor and each 'chained' visitor."""
-        
+
         v = self
         while v:
             yield v
@@ -108,9 +112,9 @@ class ClauseVisitor(object):
 
     def chain(self, visitor):
         """'chain' an additional ClauseVisitor onto this ClauseVisitor.
-        
+
         the chained visitor will receive all visit events after this one.
-        
+
         """
         tail = list(self._visitor_iterator)[-1]
         tail._next = visitor
@@ -119,7 +123,7 @@ class ClauseVisitor(object):
 class CloningVisitor(ClauseVisitor):
     """Base class for visitor objects which can traverse using 
     the cloned_traverse() function.
-    
+
     """
 
     def copy_and_process(self, list_):
@@ -135,12 +139,12 @@ class CloningVisitor(ClauseVisitor):
 class ReplacingCloningVisitor(CloningVisitor):
     """Base class for visitor objects which can traverse using 
     the replacement_traverse() function.
-    
+
     """
 
     def replace(self, elem):
         """receive pre-copied elements during a cloning traversal.
-        
+
         If the method returns a new element, the element is used 
         instead of creating a simple copy of the element.  Traversal 
         will halt on the newly returned element if it is re-encountered.
@@ -153,15 +157,15 @@ class ReplacingCloningVisitor(CloningVisitor):
         def replace(elem):
             for v in self._visitor_iterator:
                 e = v.replace(elem)
-                if e:
+                if e is not None:
                     return e
         return replacement_traverse(obj, self.__traverse_options__, replace)
 
 def iterate(obj, opts):
     """traverse the given expression structure, returning an iterator.
-    
+
     traversal is configured to be breadth-first.
-    
+
     """
     stack = deque([obj])
     while stack:
@@ -172,9 +176,9 @@ def iterate(obj, opts):
 
 def iterate_depthfirst(obj, opts):
     """traverse the given expression structure, returning an iterator.
-    
+
     traversal is configured to be depth-first.
-    
+
     """
     stack = deque([obj])
     traversal = deque()
@@ -193,7 +197,7 @@ def traverse_using(iterator, obj, visitors):
         if meth:
             meth(target)
     return obj
-    
+
 def traverse(obj, opts, visitors):
     """traverse and visit the given expression structure using the default iterator."""
 
@@ -206,7 +210,7 @@ def traverse_depthfirst(obj, opts, visitors):
 
 def cloned_traverse(obj, opts, visitors):
     """clone the given expression structure, allowing modifications by visitors."""
-    
+
     cloned = util.column_dict()
 
     def clone(element):
@@ -233,13 +237,13 @@ def cloned_traverse(obj, opts, visitors):
 
 def replacement_traverse(obj, opts, replace):
     """clone the given expression structure, allowing element replacement by a given replacement function."""
-    
+
     cloned = util.column_dict()
     stop_on = util.column_set(opts.get('stop_on', []))
 
     def clone(element):
         newelem = replace(element)
-        if newelem:
+        if newelem is not None:
             stop_on.add(newelem)
             return newelem
 
