@@ -1,5 +1,5 @@
-# types.py
-# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# sqlalchemy/types.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -34,20 +34,20 @@ from sqlalchemy.sql.visitors import Visitable
 from sqlalchemy import util
 from sqlalchemy import processors
 import collections
+default = util.importlater("sqlalchemy.engine", "default")
 
-DefaultDialect = None
 NoneType = type(None)
 if util.jython:
     import array
 
 class AbstractType(Visitable):
-    
+
     def copy_value(self, value):
         return value
 
     def bind_processor(self, dialect):
         """Defines a bind parameter processing function.
-        
+
         :param dialect: Dialect instance in use.
 
         """
@@ -56,11 +56,11 @@ class AbstractType(Visitable):
 
     def result_processor(self, dialect, coltype):
         """Defines a result-column processing function.
-        
+
         :param dialect: Dialect instance in use.
 
         :param coltype: DBAPI coltype argument received in cursor.description.
-        
+
         """
 
         return None
@@ -78,18 +78,18 @@ class AbstractType(Visitable):
         objects alone.  Values such as dicts, lists which
         are serialized into strings are examples of "mutable" 
         column structures.
-        
+
         When this method is overridden, :meth:`copy_value` should
         also be supplied.   The :class:`.MutableType` mixin
         is recommended as a helper.
-        
+
         """
         return False
 
     def get_dbapi_type(self, dbapi):
         """Return the corresponding type object from the underlying DB-API, if
         any.
-        
+
          This can be useful for calling ``setinputsizes()``, for example.
 
         """
@@ -98,10 +98,10 @@ class AbstractType(Visitable):
     def _adapt_expression(self, op, othertype):
         """evaluate the return type of <self> <op> <othertype>,
         and apply any adaptations to the given operator.
-        
+
         """
         return op, self
-        
+
     @util.memoized_property
     def _type_affinity(self):
         """Return a rudimental 'affinity' value expressing the general class
@@ -115,27 +115,46 @@ class AbstractType(Visitable):
                 typ = t
         else:
             return self.__class__
-    
+
     def _coerce_compared_value(self, op, value):
+        """Suggest a type for a 'coerced' Python value in an expression.
+        
+        Given an operator and value, gives the type a chance
+        to return a type which the value should be coerced into.
+        
+        The default behavior here is conservative; if the right-hand
+        side is already coerced into a SQL type based on its 
+        Python type, it is usually left alone.
+        
+        End-user functionality extension here should generally be via
+        :class:`.TypeDecorator`, which provides more liberal behavior in that
+        it defaults to coercing the other side of the expression into this
+        type, thus applying special Python conversions above and beyond those
+        needed by the DBAPI to both ides. It also provides the public method
+        :meth:`.TypeDecorator.coerce_compared_value` which is intended for
+        end-user customization of this behavior.
+        
+        """
+
         _coerced_type = type_map.get(type(value), NULLTYPE)
         if _coerced_type is NULLTYPE or _coerced_type._type_affinity \
             is self._type_affinity:
             return self
         else:
             return _coerced_type
-        
+
     def _compare_type_affinity(self, other):
         return self._type_affinity is other._type_affinity
 
     def compile(self, dialect=None):
         # arg, return value is inconsistent with
         # ClauseElement.compile()....this is a mistake.
-        
+
         if not dialect:
             dialect = self._default_dialect
-        
+
         return dialect.type_compiler.process(self)
-    
+
     @property
     def _default_dialect(self):
         if self.__class__.__module__.startswith("sqlalchemy.dialects"):
@@ -143,11 +162,8 @@ class AbstractType(Visitable):
             mod = ".".join(tokens)
             return getattr(__import__(mod).dialects, tokens[-1]).dialect()
         else:
-            global DefaultDialect
-            if DefaultDialect is None:
-                from sqlalchemy.engine.default import DefaultDialect
-            return DefaultDialect()
-        
+            return default.DefaultDialect()
+
     def __str__(self):
         # Py3K
         #return unicode(self.compile())
@@ -253,24 +269,24 @@ class UserDefinedType(TypeEngine):
     def _adapt_expression(self, op, othertype):
         """evaluate the return type of <self> <op> <othertype>,
         and apply any adaptations to the given operator.
-        
+
         """
         return self.adapt_operator(op), self
 
     def adapt_operator(self, op):
         """A hook which allows the given operator to be adapted
         to something new. 
-        
+
         See also UserDefinedType._adapt_expression(), an as-yet-
         semi-public method with greater capability in this regard.
-        
+
         """
         return op
 
 class TypeDecorator(AbstractType):
     """Allows the creation of types which add additional functionality
     to an existing type.
-    
+
     This method is preferred to direct subclassing of SQLAlchemy's
     built-in types as it ensures that all required functionality of 
     the underlying type is kept in place.
@@ -285,7 +301,7 @@ class TypeDecorator(AbstractType):
           '''
 
           impl = types.Unicode
-          
+
           def process_bind_param(self, value, dialect):
               return "PREFIX:" + value
 
@@ -306,40 +322,40 @@ class TypeDecorator(AbstractType):
     method. This is used to give the expression system a hint when coercing
     Python objects into bind parameters within expressions. Consider this
     expression::
-    
+
         mytable.c.somecol + datetime.date(2009, 5, 15)
-        
+
     Above, if "somecol" is an ``Integer`` variant, it makes sense that 
     we're doing date arithmetic, where above is usually interpreted
     by databases as adding a number of days to the given date. 
     The expression system does the right thing by not attempting to
     coerce the "date()" value into an integer-oriented bind parameter.
-    
+
     However, in the case of ``TypeDecorator``, we are usually changing an
     incoming Python type to something new - ``TypeDecorator`` by default will
     "coerce" the non-typed side to be the same type as itself. Such as below,
     we define an "epoch" type that stores a date value as an integer::
-    
+
         class MyEpochType(types.TypeDecorator):
             impl = types.Integer
-            
+
             epoch = datetime.date(1970, 1, 1)
-            
+
             def process_bind_param(self, value, dialect):
                 return (value - self.epoch).days
-            
+
             def process_result_value(self, value, dialect):
                 return self.epoch + timedelta(days=value)
 
     Our expression of ``somecol + date`` with the above type will coerce the
-    "date" on the right side to also be treated as ``MyEpochType``.  
-    
+    "date" on the right side to also be treated as ``MyEpochType``.
+
     This behavior can be overridden via the
     :meth:`~TypeDecorator.coerce_compared_value` method, which returns a type
     that should be used for the value of the expression. Below we set it such
     that an integer value will be treated as an ``Integer``, and any other
     value is assumed to be a date and will be treated as a ``MyEpochType``::
-    
+
         def coerce_compared_value(self, op, value):
             if isinstance(value, int):
                 return Integer()
@@ -357,10 +373,10 @@ class TypeDecorator(AbstractType):
                                  "'impl' which refers to the class of "
                                  "type being decorated")
         self.impl = to_instance(self.__class__.impl, *args, **kwargs)
-    
+
     def adapt(self, cls):
         return cls()
-        
+
     def dialect_impl(self, dialect):
         key = (dialect.__class__, dialect.server_version_info)
 
@@ -398,7 +414,7 @@ class TypeDecorator(AbstractType):
 
     def type_engine(self, dialect):
         """Return a TypeEngine instance for this TypeDecorator.
-        
+
         """
         adapted = dialect.type_descriptor(self)
         if adapted is not self:
@@ -463,28 +479,30 @@ class TypeDecorator(AbstractType):
             return process
         else:
             return self.impl.result_processor(dialect, coltype)
-    
+
     def coerce_compared_value(self, op, value):
         """Suggest a type for a 'coerced' Python value in an expression.
-        
+
         By default, returns self.   This method is called by
         the expression system when an object using this type is 
         on the left or right side of an expression against a plain Python
         object which does not yet have a SQLAlchemy type assigned::
-        
+
             expr = table.c.somecolumn + 35
-            
+
         Where above, if ``somecolumn`` uses this type, this method will
         be called with the value ``operator.add``
         and ``35``.  The return value is whatever SQLAlchemy type should
         be used for ``35`` for this particular operation.
-        
+
         """
         return self
 
     def _coerce_compared_value(self, op, value):
+        """See :meth:`.AbstractType._coerce_compared_value` for a description."""
+
         return self.coerce_compared_value(op, value)
-        
+
     def copy(self):
         instance = self.__class__.__new__(self.__class__)
         instance.__dict__.update(self.__dict__)
@@ -508,11 +526,11 @@ class TypeDecorator(AbstractType):
         objects alone.  Values such as dicts, lists which
         are serialized into strings are examples of "mutable" 
         column structures.
-        
+
         When this method is overridden, :meth:`copy_value` should
         also be supplied.   The :class:`.MutableType` mixin
         is recommended as a helper.
-        
+
         """
         return self.impl.is_mutable()
 
@@ -534,7 +552,7 @@ class MutableType(object):
     which applies special rules to such values in order to guarantee 
     that changes are detected.  These rules may have a significant 
     performance impact, described below.
-    
+
     A :class:`MutableType` usually allows a flag called
     ``mutable=True`` to enable/disable the "mutability" flag,
     represented on this class by :meth:`is_mutable`.  Examples 
@@ -542,7 +560,7 @@ class MutableType(object):
     :class:`~sqlalchemy.dialects.postgresql.base.ARRAY`.  Setting
     this flag to ``False`` effectively disables any mutability-
     specific behavior by the ORM.
-    
+
     :meth:`copy_value` and :meth:`compare_values` represent a copy
     and compare function for values of this type - implementing
     subclasses should override these appropriately.
@@ -561,29 +579,29 @@ class MutableType(object):
     execution of :class:`Query` will require a full scan of that subset of
     the 6000 objects that have mutable attributes, possibly resulting
     in tens of thousands of additional method calls for every query.
-    
+
     Note that for small numbers (< 100 in the Session at a time)
     of objects with "mutable" values, the performance degradation is 
     negligible.  In most cases it's likely that the convenience allowed 
     by "mutable" change detection outweighs the performance penalty.
-    
+
     It is perfectly fine to represent "mutable" data types with the
     "mutable" flag set to False, which eliminates any performance
     issues. It means that the ORM will only reliably detect changes
     for values of this type if a newly modified value is of a different 
     identity (i.e., ``id(value)``) than what was present before - 
     i.e., instead of operations like these::
-    
+
         myobject.somedict['foo'] = 'bar'
         myobject.someset.add('bar')
         myobject.somelist.append('bar')
-        
+
     You'd instead say::
-    
+
         myobject.somevalue = {'foo':'bar'}
         myobject.someset = myobject.someset.union(['bar'])
         myobject.somelist = myobject.somelist + ['bar']
-        
+
     A future release of SQLAlchemy will include instrumented
     collection support for mutable types, such that at least usage of
     plain Python datastructures will be able to emit events for
@@ -594,10 +612,10 @@ class MutableType(object):
 
     def is_mutable(self):
         """Return True if the target Python type is 'mutable'.
-        
+
         For :class:`.MutableType`, this method is set to 
         return ``True``.
-        
+
         """
         return True
 
@@ -675,13 +693,13 @@ class Concatenable(object):
 
 class _DateAffinity(object):
     """Mixin date/time specific expression adaptations.
-    
+
     Rules are implemented within Date,Time,Interval,DateTime, Numeric,
     Integer. Based on http://www.postgresql.org/docs/current/static
     /functions-datetime.html.
-    
+
     """
-    
+
     @property
     def _expression_adaptations(self):
         raise NotImplementedError()
@@ -740,12 +758,12 @@ class String(Concatenable, TypeEngine):
           for all String types by setting 
           :attr:`sqlalchemy.engine.base.Dialect.convert_unicode`
           on create_engine().
-          
+
           To instruct SQLAlchemy to perform Unicode encoding/decoding
           even on a platform that already handles Unicode natively,
           set convert_unicode='force'.  This will incur significant
           performance overhead when fetching unicode result columns.
-          
+
         :param assert_unicode: Deprecated.  A warning is raised in all cases
           when a non-Unicode object is passed when SQLAlchemy would coerce
           into an encoding (note: but **not** when the DBAPI handles unicode
@@ -770,7 +788,7 @@ class String(Concatenable, TypeEngine):
         if unicode_error is not None and convert_unicode != 'force':
             raise exc.ArgumentError("convert_unicode must be 'force' "
                                         "when unicode_error is set.")
-        
+
         if assert_unicode:
             util.warn_deprecated('assert_unicode is deprecated. '
                                  'SQLAlchemy emits a warning in all '
@@ -784,7 +802,7 @@ class String(Concatenable, TypeEngine):
         self.convert_unicode = convert_unicode
         self.unicode_error = unicode_error
         self._warn_on_bytestring = _warn_on_bytestring
-        
+
     def adapt(self, impltype):
         return impltype(
                     length=self.length,
@@ -812,10 +830,11 @@ class String(Concatenable, TypeEngine):
                     return None
             else:
                 encoder = codecs.getencoder(dialect.encoding)
+                warn_on_bytestring = self._warn_on_bytestring
                 def process(value):
                     if isinstance(value, unicode):
                         return encoder(value, self.unicode_error)[0]
-                    elif value is not None:
+                    elif warn_on_bytestring and value is not None:
                         util.warn("Unicode type received non-unicode bind "
                                   "param value")
                     return value
@@ -828,16 +847,16 @@ class String(Concatenable, TypeEngine):
         needs_convert = wants_unicode and \
                         (dialect.returns_unicode_strings is not True or 
                         self.convert_unicode == 'force')
-       
+
         if needs_convert:
             to_unicode = processors.to_unicode_processor_factory(
                                     dialect.encoding, self.unicode_error)
-            
+
             if dialect.returns_unicode_strings:
                 # we wouldn't be here unless convert_unicode='force'
                 # was specified, or the driver has erratic unicode-returning
                 # habits.  since we will be getting back unicode
-                # in most cases, we check for it (decode will fail).   
+                # in most cases, we check for it (decode will fail).
                 def process(value):
                     if isinstance(value, unicode):
                         return value
@@ -872,7 +891,7 @@ class Unicode(String):
     ``u'somevalue'``) into encoded bytestrings when passing the value
     to the database driver, and similarly decodes values from the
     database back into Python ``unicode`` objects.
-    
+
     It's roughly equivalent to using a ``String`` object with
     ``convert_unicode=True``, however
     the type has other significances in that it implies the usage 
@@ -880,7 +899,7 @@ class Unicode(String):
     This may affect what type is emitted when issuing CREATE TABLE
     and also may effect some DBAPI-specific details, such as type
     information passed along to ``setinputsizes()``.
-    
+
     When using the ``Unicode`` type, it is only appropriate to pass
     Python ``unicode`` objects, and not plain ``str``.  If a
     bytestring (``str``) is passed, a runtime warning is issued.  If
@@ -900,7 +919,7 @@ class Unicode(String):
     """
 
     __visit_name__ = 'unicode'
-    
+
     def __init__(self, length=None, **kwargs):
         """
         Create a Unicode-converting String type.
@@ -911,10 +930,10 @@ class Unicode(String):
           *length* for use in DDL, and will raise an exception when
           the ``CREATE TABLE`` DDL is issued.  Whether the value is
           interpreted as bytes or characters is database specific.
-        
+
         :param \**kwargs: passed through to the underlying ``String``
           type.
-          
+
         """
         kwargs.setdefault('convert_unicode', True)
         kwargs.setdefault('_warn_on_bytestring', True)
@@ -957,7 +976,7 @@ class Integer(_DateAffinity, TypeEngine):
 
     def get_dbapi_type(self, dbapi):
         return dbapi.NUMBER
-    
+
     @util.memoized_property
     def _expression_adaptations(self):
         # TODO: need a dictionary object that will
@@ -1033,14 +1052,14 @@ class Numeric(_DateAffinity, TypeEngine):
           values should be sent as Python Decimal objects, or
           as floats.   Different DBAPIs send one or the other based on
           datatypes - the Numeric type will ensure that return values
-          are one or the other across DBAPIs consistently.  
-          
+          are one or the other across DBAPIs consistently.
+
         When using the ``Numeric`` type, care should be taken to ensure
         that the asdecimal setting is apppropriate for the DBAPI in use -
         when Numeric applies a conversion from Decimal->float or float->
         Decimal, this conversion incurs an additional performance overhead
         for all result columns received. 
-        
+
         DBAPIs that return Decimal natively (e.g. psycopg2) will have 
         better accuracy and higher performance with a setting of ``True``,
         as the native translation to Decimal reduces the amount of floating-
@@ -1084,7 +1103,7 @@ class Numeric(_DateAffinity, TypeEngine):
                           'consider storing Decimal numbers as strings '
                           'or integers on this platform for lossless '
                           'storage.' % (dialect.name, dialect.driver))
-                
+
                 # we're a "numeric", DBAPI returns floats, convert.
                 if self.scale is not None:
                     return processors.to_decimal_processor_factory(
@@ -1127,11 +1146,11 @@ class Numeric(_DateAffinity, TypeEngine):
         }
 
 class Float(Numeric):
-    """A type for ``float`` numbers.  
-    
+    """A type for ``float`` numbers.
+
     Returns Python ``float`` objects by default, applying
     conversion as needed.
-    
+
     """
 
     __visit_name__ = 'float'
@@ -1142,7 +1161,7 @@ class Float(Numeric):
 
         :param precision: the numeric precision for use in DDL ``CREATE
            TABLE``.
-        
+
         :param asdecimal: the same flag as that of :class:`Numeric`, but
           defaults to ``False``.   Note that setting this flag to ``True``
           results in floating point conversion.
@@ -1217,7 +1236,7 @@ class DateTime(_DateAffinity, TypeEngine):
                 DateTime:Interval,
             },
         }
-        
+
 
 class Date(_DateAffinity,TypeEngine):
     """A type for ``datetime.date()`` objects."""
@@ -1238,12 +1257,12 @@ class Date(_DateAffinity,TypeEngine):
             operators.sub:{
                 # date - integer = date
                 Integer:Date,
-                
+
                 # date - date = integer.
                 Date:Integer,
 
                 Interval:DateTime,
-                
+
                 # date - datetime = interval,
                 # this one is not in the PG docs 
                 # but works
@@ -1316,17 +1335,19 @@ class _Binary(TypeEngine):
     # end Py2K
 
     def _coerce_compared_value(self, op, value):
+        """See :meth:`.AbstractType._coerce_compared_value` for a description."""
+
         if isinstance(value, basestring):
             return self
         else:
             return super(_Binary, self)._coerce_compared_value(op, value)
-    
+
     def adapt(self, impltype):
         return impltype(length=self.length)
 
     def get_dbapi_type(self, dbapi):
         return dbapi.BINARY
-    
+
 class LargeBinary(_Binary):
     """A type for large binary byte data.
 
@@ -1356,7 +1377,7 @@ class LargeBinary(_Binary):
 
 class Binary(LargeBinary):
     """Deprecated.  Renamed to LargeBinary."""
-    
+
     def __init__(self, *arg, **kw):
         util.warn_deprecated('The Binary type has been renamed to '
                              'LargeBinary.')
@@ -1364,13 +1385,13 @@ class Binary(LargeBinary):
 
 class SchemaType(object):
     """Mark a type as possibly requiring schema-level DDL for usage.
-    
+
     Supports types that must be explicitly created/dropped (i.e. PG ENUM type)
     as well as types that are complimented by table or schema level
     constraints, triggers, and other rules.
-    
+
     """
-    
+
     def __init__(self, **kw):
         self.name = kw.pop('name', None)
         self.quote = kw.pop('quote', None)
@@ -1381,10 +1402,10 @@ class SchemaType(object):
                     util.portable_instancemethod(self._on_metadata_create))
             self.metadata.append_ddl_listener('after-drop',
                     util.portable_instancemethod(self._on_metadata_drop))
-            
+
     def _set_parent(self, column):
         column._on_table_attach(util.portable_instancemethod(self._set_table))
-        
+
     def _set_table(self, table, column):
         table.append_ddl_listener('before-create',
                                   util.portable_instancemethod(
@@ -1397,14 +1418,14 @@ class SchemaType(object):
                     util.portable_instancemethod(self._on_metadata_create))
             table.metadata.append_ddl_listener('after-drop',
                     util.portable_instancemethod(self._on_metadata_drop))
-    
+
     @property
     def bind(self):
         return self.metadata and self.metadata.bind or None
-        
+
     def create(self, bind=None, checkfirst=False):
         """Issue CREATE ddl for this type, if applicable."""
-        
+
         if bind is None:
             bind = schema._bind_or_error(self)
         t = self.dialect_impl(bind.dialect)
@@ -1419,7 +1440,7 @@ class SchemaType(object):
         t = self.dialect_impl(bind.dialect)
         if t is not self and isinstance(t, SchemaType):
             t.drop(bind=bind, checkfirst=checkfirst)
-        
+
     def _on_table_create(self, event, target, bind, **kw):
         t = self.dialect_impl(bind.dialect)
         if t is not self and isinstance(t, SchemaType):
@@ -1439,22 +1460,22 @@ class SchemaType(object):
         t = self.dialect_impl(bind.dialect)
         if t is not self and isinstance(t, SchemaType):
             t._on_metadata_drop(event, target, bind, **kw)
-    
+
 class Enum(String, SchemaType):
     """Generic Enum Type.
-    
+
     The Enum type provides a set of possible string values which the 
     column is constrained towards.
-    
+
     By default, uses the backend's native ENUM type if available, 
     else uses VARCHAR + a CHECK constraint.
     """
-    
+
     __visit_name__ = 'enum'
-    
+
     def __init__(self, *enums, **kw):
         """Construct an enum.
-        
+
         Keyword arguments which don't apply to a specific backend are ignored
         by that backend.
 
@@ -1506,7 +1527,7 @@ class Enum(String, SchemaType):
                     break
             else:
                 convert_unicode = False
-        
+
         if self.enums:
             length =max(len(x) for x in self.enums)
         else:
@@ -1520,11 +1541,11 @@ class Enum(String, SchemaType):
     def _should_create_constraint(self, compiler):
         return not self.native_enum or \
                     not compiler.dialect.supports_native_enum
-    
+
     def _set_table(self, table, column):
         if self.native_enum:
             SchemaType._set_table(self, table, column)
-            
+
 
         e = schema.CheckConstraint(
                         column.in_(self.enums),
@@ -1533,7 +1554,7 @@ class Enum(String, SchemaType):
                                         self._should_create_constraint)
                     )
         table.append_constraint(e)
-        
+
     def adapt(self, impltype):
         if issubclass(impltype, Enum):
             return impltype(name=self.name, 
@@ -1557,7 +1578,7 @@ class PickleType(MutableType, TypeDecorator):
 
     **Note:** be sure to read the notes for :class:`MutableType` regarding
     ORM performance implications.
-    
+
     """
 
     impl = LargeBinary
@@ -1577,7 +1598,7 @@ class PickleType(MutableType, TypeDecorator):
           :meth:`AbstractType.is_mutable`.   When ``True``, incoming
           objects should provide an ``__eq__()`` method which
           performs the desired deep comparison of members, or the
-          ``comparator`` argument must be present.  
+          ``comparator`` argument must be present.
 
         :param comparator: optional. a 2-arg callable predicate used
           to compare values of this type.  Otherwise, 
@@ -1637,11 +1658,11 @@ class PickleType(MutableType, TypeDecorator):
 
     def is_mutable(self):
         """Return True if the target Python type is 'mutable'.
-        
+
         When this method is overridden, :meth:`copy_value` should
         also be supplied.   The :class:`.MutableType` mixin
         is recommended as a helper.
-        
+
         """
         return self.mutable
 
@@ -1658,25 +1679,25 @@ class Boolean(TypeEngine, SchemaType):
 
     def __init__(self, create_constraint=True, name=None):
         """Construct a Boolean.
-        
+
         :param create_constraint: defaults to True.  If the boolean 
           is generated as an int/smallint, also create a CHECK constraint
           on the table that ensures 1 or 0 as a value.
-        
+
         :param name: if a CHECK constraint is generated, specify
           the name of the constraint.
-        
+
         """
         self.create_constraint = create_constraint
         self.name = name
-    
+
     def _should_create_constraint(self, compiler):
         return not compiler.dialect.supports_native_boolean
-        
+
     def _set_table(self, table, column):
         if not self.create_constraint:
             return
-            
+
         e = schema.CheckConstraint(
                         column.in_([0, 1]),
                         name=self.name,
@@ -1684,7 +1705,13 @@ class Boolean(TypeEngine, SchemaType):
                                     self._should_create_constraint)
                     )
         table.append_constraint(e)
-    
+
+    def bind_processor(self, dialect):
+        if dialect.supports_native_boolean:
+            return None
+        else:
+            return processors.boolean_to_int
+
     def result_processor(self, dialect, coltype):
         if dialect.supports_native_boolean:
             return None
@@ -1705,7 +1732,7 @@ class Interval(_DateAffinity, TypeDecorator):
     (such as, conversion of both sides into integer epoch values first) which
     currently is a manual procedure (such as via
     :attr:`~sqlalchemy.sql.expression.func`).
-    
+
     """
 
     impl = DateTime
@@ -1715,20 +1742,20 @@ class Interval(_DateAffinity, TypeDecorator):
                         second_precision=None, 
                         day_precision=None):
         """Construct an Interval object.
-        
+
         :param native: when True, use the actual
           INTERVAL type provided by the database, if
-          supported (currently Postgresql, Oracle).  
+          supported (currently Postgresql, Oracle).
           Otherwise, represent the interval data as 
           an epoch value regardless.
-        
+
         :param second_precision: For native interval types
           which support a "fractional seconds precision" parameter,
           i.e. Oracle and Postgresql
-        
+
         :param day_precision: for native interval types which 
           support a "day precision" parameter, i.e. Oracle.
-        
+
         """
         super(Interval, self).__init__()
         self.native = native
@@ -1740,7 +1767,7 @@ class Interval(_DateAffinity, TypeDecorator):
             return cls._adapt_from_generic_interval(self)
         else:
             return self
-    
+
     def bind_processor(self, dialect):
         impl_processor = self.impl.bind_processor(dialect)
         epoch = self.epoch
@@ -1802,6 +1829,8 @@ class Interval(_DateAffinity, TypeDecorator):
         return Interval
 
     def _coerce_compared_value(self, op, value):
+        """See :meth:`.AbstractType._coerce_compared_value` for a description."""
+
         return self.impl._coerce_compared_value(op, value)
 
 
