@@ -77,7 +77,7 @@ class TransactionTest(TestBase):
         result = connection.execute("select * from query_users")
         assert len(result.fetchall()) == 0
         connection.close()
-    
+
     def test_transaction_container(self):
 
         def go(conn, table, data):
@@ -93,7 +93,7 @@ class TransactionTest(TestBase):
                       {'user_id': 1, 'user_name': 'user3'}])
         eq_(testing.db.execute(users.select()).fetchall(), [(1, 'user1'
             )])
-        
+
     def test_nested_rollback(self):
         connection = testing.db.connect()
         try:
@@ -138,7 +138,7 @@ class TransactionTest(TestBase):
             eq_(connection.scalar("select count(*) from query_users"), 0)
         finally:
             connection.close()
-        
+
     def test_nesting(self):
         connection = testing.db.connect()
         transaction = connection.begin()
@@ -383,7 +383,7 @@ class AutoRollbackTest(TestBase):
 class ExplicitAutoCommitTest(TestBase):
 
     """test the 'autocommit' flag on select() and text() objects.
-    
+
     Requires PostgreSQL so that we may define a custom function which
     modifies the database. """
 
@@ -539,6 +539,42 @@ class TLTransactionTest(TestBase):
 
         tlengine.close()
 
+    def test_rollback_no_trans(self):
+        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+
+        # shouldn't fail
+        tlengine.rollback()
+
+        tlengine.begin()
+        tlengine.rollback()
+
+        # shouldn't fail
+        tlengine.rollback()
+
+    def test_commit_no_trans(self):
+        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+
+        # shouldn't fail
+        tlengine.commit()
+
+        tlengine.begin()
+        tlengine.rollback()
+
+        # shouldn't fail
+        tlengine.commit()
+
+    def test_prepare_no_trans(self):
+        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+
+        # shouldn't fail
+        tlengine.prepare()
+
+        tlengine.begin()
+        tlengine.rollback()
+
+        # shouldn't fail
+        tlengine.prepare()
+
     def test_connection_close(self):
         """test that when connections are closed for real, transactions
         are rolled back and disposed."""
@@ -602,6 +638,27 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
+    def test_with_interface(self):
+        trans = tlengine.begin()
+        tlengine.execute(users.insert(), user_id=1, user_name='user1')
+        tlengine.execute(users.insert(), user_id=2, user_name='user2')
+        trans.commit()
+
+        trans = tlengine.begin()
+        tlengine.execute(users.insert(), user_id=3, user_name='user3')
+        trans.__exit__(Exception, "fake", None)
+        trans = tlengine.begin()
+        tlengine.execute(users.insert(), user_id=4, user_name='user4')
+        trans.__exit__(None, None, None)
+        eq_(
+            tlengine.execute(users.select().order_by(users.c.user_id)).fetchall(),
+            [
+                (1, 'user1'),
+                (2, 'user2'),
+                (4, 'user4'),
+            ]
+        )
+
     def test_commits(self):
         connection = tlengine.connect()
         assert connection.execute('select count(*) from query_users'
@@ -954,11 +1011,11 @@ class ForUpdateTest(TestBase):
     @testing.requires.independent_connections
     def test_queued_update(self):
         """Test SELECT FOR UPDATE with concurrent modifications.
-        
+
         Runs concurrent modifications on a single row in the users
         table, with each mutator trying to increment a value stored in
         user_name.
-        
+
         """
 
         db = testing.db
